; JCM-1 Monitor ROM
; Version 0.1
; Copyright: Joe Strosnider
; Episode 5 Code Date: 1/28/2018 - 2/4/2018
;	Original Features
;
; Episode 7 Code Date: 5/15/2018
;	Added Startup Chime
;	Added Error Beep
;	Improved space/performance of using M_IsHex
;	Other minor changes and improvements
;
; Purpose: A simple monitor ROM we can use to input, view and execute code.
; Assembler Syntax: Retro Assembler - https://enginedesigns.net/apps/

.target "65C02", 64	; We're running a 65C02 Processor


/* ******************************************************** */
/* Constants												*/

	/* Zero Page Locations */
		; Monitor Locations 	($00 - $3F)
			CmdBufPtr	= $00	; Means two things simultaneously.
									;   1) Points to the next open space in the command buffer.
									;   2) Is equal to the length of buffer contents.
										
			StrOutPtr	= $01	; Pointer to string location

			MonCurLo	= $03	; Low byte of monitor cursor location
			MonCurHi	= $04	; High byte of monitor cursor location
			
			ADIndex		= $05	; scratch-pad of the loop location of the Address Decoder
			
			StorCurLo	= $06	; Low byte of Store routine location
			StorCurHi	= $07	; High byte of Store routine location
			SMIndex		= $08	; byte index byte parser in Store routine
		
		; BIOS ROM Locations	($40 - $7F)
			Sound_ChipSelect	= $40	; Holds which sound chip we're talking to.
			Sound_Register		= $41	; Holds the register we're modifying.
			Sound_Value			= $42	; Holds the sound value we're sending to the register.
		
	/* Other General Locations */
		CmdBuf		= $0200
			
	/* I/O Locations */
		; ACIA 0
			ACIA0_Data			= $D000 
			ACIA0_Status 		= $D001 
			ACIA0_Command 		= $D002 
			ACIA0_Control 		= $D003
		
		; Sound Control VIA
			Sound_VIA			= $D020
			Sound_OutRegB		= Sound_VIA
			Sound_DDRB	 		= Sound_VIA+2
			Sound_DDRA	 		= Sound_VIA+3
			Sound_OutRegA		= Sound_VIA+15
			Sound_SR			= $D02A
			Sound_ACR			= $D02B
			Sound_IFR			= $D02D
			Sound_IER			= $D02E
/* ******************************************************** */

	
/* ******************************************************** */
/* Assembler Trick to Start ROM at $0000					*/
/* You can remove the following three lines if your ROM		*/
/* programmer allows you to save the data at $E000 offset	*/
	* = $0000
	.Loop $DFFF
	.byte $ff
	.endloop

	
/* ******************************************************** */
/* ROM Program Starts Here									*/

* = $e000
BIOS_ColdBoot	; Sets hardware to power on defaults, plays the Startup Chime, "ANSI Clears" screen and says hello.
	JSR InitACIA0	; Get ACIA0 Ready for transmission.
	JSR StartChime	; Sing a pretty song.
	JSR M_PrintImm	; Say hello.
	.byte $1B,"[2J",$1B,"[H","JCM-1 Computer",$00
	
/* ******************************************************** */
M_InitMonitor		; do stuff entering the monitor for the first time.
	STZ MonCurLo	; 
	LDA #$E0		; Initialize the monitor cursor location.
	STA MonCurHi	;
	
/* ******************************************************** */	
M_CmdPrompt		; Show the command prompt.
	STZ CmdBufPtr				; Set the command buffer pointer to zero.
	
	JSR M_PrintImm				; Advance two lines.
	.byte $0D,$0A,$0D,$0A,$00	
	
	JSR M_PtMonCur				; Print Monitor Cursor
	LDA #">"					; Load up a bracket..
	JSR ACIA0_SendByte			; send the command prompt		

/* ******************************************************** */
M_Input		; Get user input. The heart of the monitor.

	JSR ACIA0_GetByte   	; Get a byte.
	CMP #$1B				; Was it Escape?
	BEQ M_CmdPrompt			; Yup. Bomb immediately to prompt.
	JSR ACIA0_SendByte  	; Otherwise, spit back the char so the user knows we got it.
	LDX CmdBufPtr			; Get the Current Buffer Length. Done here to save bytes later.
	CMP #$0D				; Is it Carriage Return?
	BEQ M_ParseCmd			; Yup. We have a command in the buffer. Go parse it.
	CMP #$08				; Was it Backspace?
	BEQ	M_BkSpc				; Yup. Go deal with that.
	
	CPX #$FF				; Is the buffer full?
	BEQ M_CmdPrompt			; Yes. Go prompt.
	STA CmdBuf,X    		; Otherwise, add input to buffer
	INC CmdBufPtr   		; Increment the buffer pointer
	BRA M_Input				; Go get another character.

M_BkSpc
	CPX #0				; Check to see if it's empty.
	BEQ M_CmdPrompt  	; Buffer empty. Go prompt.
	DEC CmdBufPtr   	; Otherwise, decrement buffer pointer
	JSR M_PrintImm		; Erase the last character on the screen.
	.byte " ",$08,$00
	BRA M_Input			; Go get another character.	

M_ParseCmd
	LDA #$0A			; Load up a line feed.
	JSR ACIA0_SendByte 	; Spit it out.
	CPX #$0				; Remember, we've already loaded this with CmdBufPtr...
	BEQ CmdListImm 		; 0 = blank input. Go list memory.
	LDX #$0				; Ready to parse. Let's start at the beginning.
	PC_Ph1	LDA CmdBuf,X	; Let's get a byte.
			CMP #$20		; Is it a space?
			BNE PC_Ph2		; Nope, continue.
			INX				; It's a space. Increment.
			CPX CmdBufPtr	; Where are we in the command buffer?
			BEQ CmdListImm 	; Reached buffer end and found nothing but spaces. Go list memory.
			BRA PC_Ph1		; Otherwise, go get another byte.
		
	PC_Ph2	AND #$DF		; Convert to uppercase.
			CMP #"G"		; [G]o?
			BEQ CmdGo  		; Yep, go execute the memory location.
			CMP #"L"		; [L]ist memory?
			BEQ	CmdList		; Yep, go list memory location
			CMP #"S"		; [S]tore Memory?
			BEQ CmdStore	; Yep, go store memory
	
	PC_Err	JSR M_PrintImm	; fell through, must be an error
			.byte "Err",$0D,$0A,$00
			JSR ErrorBeep
			BRA M_CmdPrompt

	CmdList	LDY CmdBufPtr		; Grab the pointer.
			DEY					; Decrement so we're looking at the last actual byte in the buffer.
			LDA CmdBuf,Y		; Grab the byte.
			AND #$DF			; Convert to uppercase.
			CMP #'L'			; Compare with "L"
			BEQ CmdListImm		; All we have in the buffer is "L". Just go show memory.
			JSR M_AddrDecode	; Otherwise, we have some hex to decode. Let's do that.
								; Then come back and list memory immediately

	CmdListImm
			JSR M_PrintImm			
			.byte $1B,"[1A",$00	; Put the cursor back where it was...
			JSR M_PtMonCur		; Show the Monitor Cursor
			LDY #0			
	LMLoop	LDA (MonCurLo),Y	; Grab a byte.
			JSR M_PtHex			; Print it out
			LDA #" "			; Give us a little space...
			JSR ACIA0_SendByte	; ... would ya, huh?
			INY					; Increment
			CPY #16				; 16 bytes?
			BNE LMLoop			; Nope, go again.
			LDA MonCurLo		; Load Monitor Cursor
			ADC #15				; Move it forward.
			STA MonCurLo		; Save it out.
			BCC LMExit			; Did it roll over?
			INC MonCurHi		; Yep. Then increment hi byte
	LMExit	JMP M_CmdPrompt

	CmdGo	JSR M_AddrDecode	; Put the Go address in MonCurLo/Hi
			JMP (MonCurLo)		; Unconditional Jump.
		
	CmdStore
			LDY CmdBufPtr	; Grab the pointer.
			DEY				; Decrement to look at the last char in the buffer.
			LDA CmdBuf,Y	; Grab the byte.
			AND #$DF		; Convert to uppercase.
			CMP #'S'		; Compare with "S"
			BEQ CSMExit		; All we have in the buffer is "S". Nothing to do. Go prompt.
			INX
			LDA CmdBuf,X	; Grab the next character.
			CMP #':'		; Is it a colon?
			BEQ CSMPhaseA	; Yes. Next thing might be bytes to store. Check that.
			DEX				
			JSR M_AddrDecode ; No. Must be an address. Go decode it
			LDA CmdBuf,X	; Grab char after the address.
			CMP #':'		; Again, is it a colon?
			BNE CSMExit		; No. Error in input.
		
		CSMPhaseA	INX				; Next Char
					CPX CmdBufPtr	; Have we reached buffer end?
					BEQ CSMExit		; Yep. No bytes to store.
					
					LDA MonCurLo	; We have bytes! Copy monitor location to store routine cursor...
					STA StorCurLo	; ..so that store routine
					LDA MonCurHi	; ....can do it's
					STA StorCurHi	; ......magic
		
		CSMPhaseB	STZ SMIndex		; Zero the byte index.
					STZ MonCurLo    ; We need a clean byte to deal with the math.
					
		CSMPhaseC	LDA CmdBuf,X	; Get a byte
					JSR M_IsHex		; Ask if it's hex
					BEQ	CSMPhaseG	; Not Hex. 
					
		CSMPhaseD	LDY SMIndex		; Which nibble are we on?
					BNE CSMPhaseE	; Second nibble.
					JSR M_ToNibble	; First nibble. Shift Accumulator into MonCurLo
					INX
					CPX CmdBufPtr	; CmdBuf End?
					BNE CSMPhaseF	; Not done. Continue.
					JSR CSMStore 	; We're done.
					LDA StorCurLo	; put the monitor where it needs to be.
					STA MonCurLo	;  "
					LDA StorCurHi	;  "
					STA MonCurHi	;  "
					BRA CSMExit 	; Go prompt
					
		CSMPhaseE	JSR M_ToNibble	; Shift Accumulator into MonCurLo	
					JSR CSMStore	; Store it
					BRA CSMPhaseH	
					
		CSMPhaseF	INC SMIndex		; Advance
					BRA CSMPhaseC	; Go again.
					
		CSMPhaseG	LDY SMIndex		; Load the Byte Index
					BEQ CSMPhaseH	; It's Zero. Need to check again.
					JSR CSMStore    ; It's one. Store previous nibble to memory.
		CSMPhaseH	LDA StorCurLo	; put the monitor where it needs to be.
					STA MonCurLo	;
					LDA StorCurHi	;
					STA MonCurHi	;
					INX
		CSMPhaseI	CPX CmdBufPtr	;Reached buffer end?
					BNE CSMPhaseB	;Nope. Keep going.
		CSMExit		JMP M_CmdPrompt	;Yup. We're done.
		
		CSMStore	LDY #0
					LDA MonCurLo		; Grab the byte to store into Accumulator
					STA (StorCurLo),Y	; store at address indicated by StorCurLo
					INC StorCurLo		; Increment StorCurLo
					BNE CSMStore2		; No rollover? Then done.
					INC StorCurHi		; Rolled over. Increment StorCurHi
		CSMStore2	RTS


/* ******************************************************** */
M_AddrDecode	; decodes the hex value at the current buffer pointer and puts it in MonCurLo/Hi
				; Inspired by Woz, heavily modified.
				
	STZ MonCurLo	; Zero out MonCur-- so nibble shift works properly.
	STZ MonCurHi	;	"
	STZ	ADIndex 	; Keeps track of times we've looped, since both index registers are taken up.
	INX				; because we're still pointing at the command char, not the value. :o)
	AD_Loop
		JSR M_IsHex 	; Is it hex?
		BEQ AD_Exit 	; Not hex. Bomb.
								
		JSR M_ToNibble	; Is Hex. Shift it into Zero Page.
		INX             ; Advance text index.
		CPX CmdBufPtr 	; Compare X with CmdBufPtr.
		BEQ AD_Exit		; Equal to buffer end. We're done.
						
	ADIndexTest			; Less than buffer end? Go check to see if we're hit the 4 char limit.
		INC ADIndex		; Advance Address Decoder index.
		LDA ADIndex		; Pop it into the accumulator
		CMP #$04		; Compare it with 4.
		BNE	AD_Loop		; if A < 4, we're definitely not done.
	AD_Exit
		RTS 			; We're done. Decoded address is in MonCurLo and MonCurHi

/* ******************************************************** */	
M_IsHex 	; Checks that a character in Accumulator is hex.
			; Valid returns original value in Accumulator
			; Invalid returns Accumulator = binary zero.

	IsHex1	; determine if it's a number nibble
		LDA CmdBuf,X
		CMP #$30 	; if < "0", go check it for letter
		BCC IsHex2	
		CMP #$3A 	; if > "9", go check it for letter (or >= 9+1)
		BCS IsHex2
		RTS			; yes number. Return with value in A. If we return with something that isn't binary zero, then it's valid

	IsHex2	;determine if it's a letter nibble.
		AND #$DF	; Convert to uppercase.
		CMP #$41 	; if < "A", skip
		BCC IsHex3
		CMP #$47 	; if > "F", skip (or >= F+1)
		BCS IsHex3
		ADC #$B9	; A little binary math to make our letter suitable for bit shifting
		RTS 		; Return with value in A. If we return with something that isn't binary zero, then it's valid

	IsHex3	
		LDA #0		; If we return with EXACTLY binary zero, it's Not Hex!
		RTS

		
/* ******************************************************** */		
M_ToNibble		;Move the nibble into position. Inspired by Woz Apple 1 Monitor.
		ASL		;Move digit to MSD of A
		ASL
		ASL
		ASL
		LDY #4  ; Have to shift four times.
		
	BitShift
		ASL				; Digit left, MSB to carry
		ROL MonCurLo	; Rotate into LSD
		ROL MonCurHi	; Rotate into MSD
		DEY				; Done 4 shifts?
		BNE	BitShift	; No, loop
		RTS
	
/* ******************************************************** */	
M_PrintImm	; Outputs the null-terminated string that is stored in memory directly after the subroutine call.
			; From: http://www.6502.org/source/io/primm.htm

		PLA					; Get the low part of "return" address
		STA StrOutPtr		; Store to pointer
		PLA					; Get the high part of "return" address
		STA StrOutPtr+1		; Store to pointer
M_PI1	LDY #1				; Accounts for pointing one short. :o)
		LDA (StrOutPtr),y	; Get the next string character
		INC StrOutPtr		; Update the pointer
		BNE M_PI2			; If not, we're pointing to next character
		INC StrOutPtr+1		; Account for page crossing
M_PI2	ORA	#0				; Set flags according to contents of Accumulator
		BEQ M_PI3			; Don't print the final NULL
		JSR ACIA0_SendByte	; Print the character.
		BRA M_PI1			; Go again.
M_PI3	INC StrOutPtr
		BNE M_PI4
		INC StrOutPtr+1		; account for page crossing
M_PI4   JMP (StrOutPtr)		; return to byte following final NULL

	
/* ******************************************************** */
M_PtMonCur	; Prints the monitor cursor location on screen.
		LDA MonCurHi		; Load MSB
		JSR M_PtHex			; Convert and send MSB
		LDA MonCurLo		; Load LSB
		JSR M_PtHex			; Convert and send LSB
		LDA #":"			; Load up your colon...
		JSR ACIA0_SendByte	; ... and empty it it out.
		RTS					; ...... Bathroom humor!

/* ******************************************************** */
M_PtHex	; Prints Accumulator directly to screen as a hex byte.
		; Author: the one and only Woz.
		; Returns with LSD in Accumulator
					
			PHA				;Save A for LSD
			LSR				; /========================-
			LSR				;/| Shift MSD ...         /												<= Look at that
			LSR				;\| ... to LSD position   \												     ASCII Art! :p
			LSR				; \========================-
			JSR	PtHex		;Output hex digit, some fall through magic...
			PLA				;Restore A
PtHex		AND	#%00001111	;Mask LSD for hex print
			ORA	#"0"		;Add "0"
			CMP	#"9"+1		;Is it a decimal digit?
			BCC	ACIA0_SendByte	;Yes! output it, with a sneaksy falltrhough.
			ADC	#6			;Add offset for letter A-F
		; Falls through. Sneaksy. :o)

/* ******************************************************** */
ACIA0_SendByte	; Sends a byte out ACIA 0
				; Byte must be in A
				; Returns with registers intact			
			PHA					; Put A on the stack
A0SB_Loop	LDA ACIA0_Status	; Get ACIA Status.
			AND #$10 			; Is TX buffer full?
			BEQ A0SB_Loop		; Yes. Go ask again.
			PLA					; No. Restore A
			STA ACIA0_Data		; And send the byte out.
			RTS

/* ******************************************************** */	
ACIA0_GetByte	; Gets a byte from ACIA 0
				; Returns with received byte in A
				LDA 	ACIA0_Status		; Get ACIA Status.
				AND 	#$08 				; Was RX buffer empty?
				BEQ 	ACIA0_GetByte		; Yes. Go ask again.
				LDA 	ACIA0_Data			; No. Gimme the byte!
				RTS

/* ******************************************************** */
InitACIA0	; Set ACIA 0 for 8,N,1,115200
				LDA 	#%00001011			;No parity. No echo. No Interrupts
				STA 	ACIA0_Command
				LDA 	#%00010000			;115,200 Baud, 8 Data bits, 1 Stop bits
				STA 	ACIA0_Control
				RTS	

/* ******************************************************** */
Sound_ClearRegs	; Zeroes all registers in both sound chips.
				; Returns with registers intact.
				
				PHA							; Push Accumulator to the Stack.
				PHX							; Push X to the Stack.
				LDA 	#$FF				; Set all pins to outputs.
				STA 	Sound_DDRA			; Set PA Pins
				STA 	Sound_DDRB			; Set PB Pins
				STZ		Sound_Value			; Holds the sound value we're sending to the register.
				LDX 	#0					; Start with Register 0
	ClrLoop		STZ 	Sound_ChipSelect 	; Select Snd0
				STX		Sound_Register		; Talk to the sound register indexed by X
				JSR 	Sound_SendCmd		; Zero the Register indexed by X
				INC 	Sound_ChipSelect 	; Select Snd1
				JSR 	Sound_SendCmd		; Clear the same register on the other chip.
				INX							; Increment
				CPX 	#16					; Have we done all 16 registers?
				BNE 	ClrLoop				; Nope! Go again.
				PLX							; Pull X from the Stack.
				PLA							; Pull Accumulator from the Stack.
				RTS

/* ******************************************************** */			
StartChime		; Plays the startup chime.
				; Destroys A and X
				
				JSR 	Sound_ClearRegs		; Clear any playing sounds.
				STZ 	Sound_ChipSelect 	; Select Snd0
	ChLoop		LDA 	#$38				; 00111000	 Disable noise and enable tone on all channels
				STA		Sound_Value
				LDA		#7					; Reg 7 - Mixer Control & I/O Enable
				STA		Sound_Register
				JSR 	Sound_SendCmd
				
				LDA 	#$10				; Amplitude Controlled by Envelope
				STA		Sound_Value
				INC		Sound_Register 		; Reg 8 - Amplitude Register Chan A
				JSR 	Sound_SendCmd
				
				INC		Sound_Register		; Reg 9 - Amplitude Register Chan B
				JSR 	Sound_SendCmd
				
				INC		Sound_Register 		; Reg 10 - Amplitude Register Chan C
				JSR 	Sound_SendCmd

				STZ		Sound_Value			; Env Period Coarse Definition
				INC		Sound_Register		; Reg 11 - Env Period Coarse Register		
				JSR 	Sound_SendCmd
				
				LDA 	#$1E    			; Env Period Fine Definition
				STA		Sound_Value			
				INC 	Sound_Register		; Reg 12 - Env Period Coarse Register
				JSR 	Sound_SendCmd
				
				LDA 	#9					; Env Shape Definition (Downward Taper)
				STA		Sound_Value
				INC 	Sound_Register		; Reg 13 - Env Shape Register
				JSR 	Sound_SendCmd
				
				LDA 	Sound_ChipSelect	; Have we done both chips?
				BNE 	Play				; Yes. Then play the sound.
				INC 	Sound_ChipSelect	; Otherwise, Select Snd1 ...
			BRA ChLoop						; ... and set up channels again.

			Play	; Play the chime. Remember, Left channel is already selected.
				;F#3
				LDA 	#$52				; Tone Freq A, Fine tune
				STA 	Sound_Value			;
				STZ		Sound_Register		; Reg 0 - Channel A Fine Tune
				JSR 	Sound_SendCmd
				LDA 	#1					; Tone Freq A, Coarse tune
				STA 	Sound_Value			;
				INC		Sound_Register		; Reg 1 - Channel A Coarse Tune
				JSR 	Sound_SendCmd
				
				;C#4
				LDA 	#$E1				; Tone Freq B, fine tune
				STA 	Sound_Value
				INC		Sound_Register		; Reg 2 - Channel B Fine Tune
				JSR 	Sound_SendCmd
				STZ		Sound_Value			; Tone Freq B, Coarse tune
				INC		Sound_Register		; Reg 3 - Channel B Coarse Tune
				JSR 	Sound_SendCmd
				
				;A#4
				LDA 	#$86				; Tone Freq c, fine tune
				STA 	Sound_Value
				INC		Sound_Register		; Reg 4 - Channel C Fine Tune
				JSR 	Sound_SendCmd
				STZ		Sound_Value			; Tone Freq B, Coarse tune
				INC		Sound_Register		; Reg 5 - Channel C Coarse Tune
				JSR 	Sound_SendCmd

				STZ Sound_ChipSelect 		; Select Snd0

				;F#4		
				LDA 	#$54				; Tone Freq A, Fine tune
				STA 	Sound_Value			;
				STZ		Sound_Register		; Reg 0 - Channel A Fine Tune
				JSR 	Sound_SendCmd
				LDA 	#1					; Tone Freq A, Coarse tune
				STA 	Sound_Value			;
				INC		Sound_Register		; Reg 1 - Channel A Coarse Tune
				JSR 	Sound_SendCmd
				
				;C#5
				LDA 	#$71				; Tone Freq B, fine tune
				STA 	Sound_Value
				INC		Sound_Register		; Reg 2 - Channel B Fine Tune
				JSR 	Sound_SendCmd
				STZ		Sound_Value			; Tone Freq B, Coarse tune
				INC		Sound_Register		; Reg 3 - Channel B Coarse Tune
				JSR 	Sound_SendCmd
				
				;A#5
				LDA 	#$43				; Tone Freq c, fine tune
				STA 	Sound_Value
				INC		Sound_Register		; Reg 4 - Channel C Fine Tune
				JSR 	Sound_SendCmd
				STZ		Sound_Value			; Tone Freq B, Coarse tune
				INC		Sound_Register		; Reg 5 - Channel C Coarse Tune
				JSR 	Sound_SendCmd
				RTS	

/* ******************************************************** */
ErrorBeep		; Protests profusely when you, the very human user, do something wrong.
				; Destroys A
				
				JSR 	Sound_ClearRegs
				STZ 	Sound_ChipSelect 	; Select Snd0
				LDA 	#$38				; 00111000 - Disable noise and enable tone on all channels
				STA		Sound_Value			;
				LDA		#7					; Reg 7 - Mixer Ctrl & I/O Enable
				STA		Sound_Register		;
				JSR 	Sound_SendCmd		;
				
				LDA 	#$10				; Amplitude Controlled by Envelope
				STA		Sound_Value			;
				INC		Sound_Register		; Reg 8 - Ampl Reg Chan A
				JSR 	Sound_SendCmd		;
					
				STZ		Sound_Value			; Env Pd Coarse Def
				LDA 	#11					; Reg 11 - Env Pd. Coarse Reg
				STA 	Sound_Register		;
				JSR 	Sound_SendCmd

				LDA 	#3  	   			; Env Pd Fine Def
				STA		Sound_Value
				INC		Sound_Register		; Reg 12 - Env Pd Fine Reg
				JSR 	Sound_SendCmd
				
				LDA 	#9					; Env Shape Def (Downward Taper)
				STA		Sound_Value			;
				INC		Sound_Register		; Reg 13 - Env Shape Reg
				JSR 	Sound_SendCmd
				
				LDA 	#$3E				; Tone Freq A, fine tune
				STA		Sound_Value
				STZ		Sound_Register		; Reg 0 - Channel A Fine Tune
				JSR 	Sound_SendCmd
				STZ		Sound_Value			; Tone Freq A, Coarse tune
				INC		Sound_Register		; Reg 1 - Channel A Coarse Tune
				JSR 	Sound_SendCmd
				RTS

/* ******************************************************** */
Sound_SendCmd	;Send sound to the selected sound chip
				;Sound_ChipSelect Holds which sound chip we're talking to.
				;Sound_Register Holds the register we're modifying.
				;Sound_Value Holds the sound value we're sending to the register.
				;Returns with registers intact.

				PHA							; Save Accumulator to Stack
	Latch		LDA 	Sound_Register		; Pull in the register we're talking to.
				STA 	Sound_OutRegA		; Push it to Sound_VIA
				LDA 	Sound_ChipSelect	; Which sound chip are we talking to?
				BNE 	Latch1				; If Snd1, go do that.
				LDA 	#3					; Must be Snd0
				BNE 	LatchIt
	Latch1		LDA 	#12					; Must be Snd1
	LatchIt		STA 	Sound_OutRegB		; Latch the Register
				STZ 	Sound_OutRegB		; Clear the Sound_VIA
				
	Write		LDA		Sound_Value			; Pull in the value for the latched register.
				STA 	Sound_OutRegA		; Push it to the Sound_VIA
				LDA 	Sound_ChipSelect	; Which sound chip are we talking to?
				BNE 	Write1				; If Snd1, go do that.
				LDA 	#2					; Must be Snd0
				BNE 	WriteIt
	Write1 		LDA 	#8					; Must be Snd1
	WriteIt		STA 	Sound_OutRegB		; Latch the Value
				STZ 	Sound_OutRegB		; Clear the Sound_VIA
				PLA							; Restore Accumulator
RTS

/* ******************************************************** */
* = $FFFA	; Vectors
.word BIOS_ColdBoot	;$FFFA/$FFFB - NMI
.word BIOS_ColdBoot	;$FFFC/$FFFD - RESET
.word BIOS_ColdBoot	;$FFFE/$FFFF - IRQ/BRK
