;JCM-1 Episode 7 Stereo Test
/*
Using the Stereo Features of JCM-1:

	All six sound channels can be indivdualy panned left or right.
	On startup, all channels are set to dead center.
	
	JCM-1 Uses the "Microchip MCP4231-103E/P 8-Bit Dual SPI Digital POT with Volatile Memory" to affect volume panning.
	
	There are three of these chips on the board.
		Chip P0 sets panning for channel A and B on Chip 0.
		Chip P1 sets panning for channel A and B on Chip 1.
		Chip P2 sets panning for Channel C on both Chip 0 and 1.
		
		Each chip has an individual /CS/ chip select line connected to Sound_VIA.
		All three chips share a SCK and SI line connected to the Sound_VIA
		The SO lines on the chips are not connected, so you can't chain commands through the chips.
	
	Programming the potentiometers:
		Basically, you select a pot chip, send it a command byte, then a data byte, then deselect the chip.
		Below are details on how this is achieved.
		
		1) Set Sound_VIA in Shift Register mode 110
		
		2) Set PB4, PB5 or PB6 on the Sound VIA to 1 to address a particular Potentiometer Chip.
			PB4 / Sending #16 to Sound_OutRegB means select Pot Chip (P0) for Sound Chip 0, Ch. A or B
			PB5 / Sending #32 to Sound_OutRegB means select Pot Chip (P1) for Sound Chip 1, Ch. A or B
			PB6 / Sending #64 to Sound_OutRegB means select Pot Chip (P2) for Both Chips Channel C
			Or... since all three chips have common data and clock lines, 
				you can in theory select more than one chip and give all of them the command simultaneously.
		
		3) Send a command byte to Sound_SR.
			This tells the selected pot chip which pot to set.
			Format of Command and Data Bytes for simple Write Commands
				A: Command Address
				-: Don't Care Bits
				D: Data Bits
				
				Command Byte 	Data Byte
				-------- 		--------
				AAAA---D 		DDDDDDDD
				3210   8 		76543210
				
				Read that carefully. That means in your first byte the LSB is a data bit for the pot value. Most of the time
					though, you can just set this to zero.
				
			Example used herein:
			#%00000011 = Select Pot 0
				For P0 and P1, Pot 0 Controls Channel A.
				For P2, Pot 0 Controls Channel C on Snd 0.
				
			#%00010011 = Select Pot 1
				For P0 and P1, Pot 1 Controls Channel B.
				For P2, Pot 1 Controls Channel C on Snd 1.
				
		4) Wait until the Shift Register is done.
			You can do that however you like. Timer? IRQ? Polling the 6522 Flags? NOP loop? Do something else for a few cycles then come back? Your choice.
			In this code example, we read the Interrupt Flags on the 65C22 for the Shift Register in a tight loop. When we know the SR is done, we return.
		
		5) Send a Stereo value byte to Sound_SR. Can be any byte value.
			$00 = "Full" Left
				...
			$80 = Dead Center
				...
			$FF = "Full" Right
		
		6) Wait until the Shift Register is done. (See above)
		
		7) Unset the PB4/5/6 to apply the value to the pot.
		
		There are other commands available, like single incr/decr of the pot and disconnecting the A or B terminals,
			(i.e. to silence a side completely). Execution of these is left as an excersise to the user. Read the data sheet for details.
*/
	
.target "65C02", 64				; We're running a 65C02 Processor
.include "JCM-1_BIOS_Ep7.inc"	; We need to know where BIOS and Monitor calls are!

PotChip 	= $81
PotSelect 	= $82
PotValue 	= $83
CurrentChannel = $84

*=$0300
				JSR 	Sound_ClearRegs		; Silence any current noise.
	
				; Set up SoundVIA to clock data to potentiometers			
				LDA 	#%00011000			; We will use Shift Out mode 110 on the 65C22
				STA 	Sound_ACR			;  ... and send that to Aux Ctrl Register
				LDA 	#$FF				; Set all pins to outputs, just in case.
				STA 	Sound_DDRB 			;  ''
	
				;Say Hello.
				JSR 	M_PrintImm	
				.byte $1B,"[2J",$1B,"[H"
				.byte $0a,$0d
				.byte "       JCM-1 Stereo Test Program",$0a,$0d
				.byte $0a,$0d
				.byte "       0) Select Chip 0",$0a,$0d
				.byte "       1) Select Chip 1",$0a,$0d
				.byte "       A) Select Channel A",$0a,$0d
				.byte "       B) Select Channel B",$0a,$0d
				.byte "       C) Select Channel C",$0a,$0d
				.byte "       Space) Toggle Stereo",$0a,$0d
				.byte ,$0a,$0d
				.byte "       -= Press ESC to Exit =-",$0a,$0d,$0a,$0d
				.byte $00
	
	; Get 'dem digitses!
GetInput
				JSR 	ACIA0_GetByte   	; Get a byte.
				CMP 	#"0"
				BEQ 	CZero				; "0" Pressed.
				CMP 	#"1"				; "1" Pressed.
				BEQ 	COne
				CMP 	#" "
				BEQ 	Toggle				; Spacebar Pressed.
				CMP 	#$1B				; Was it Escape?
				BEQ 	GoPrompt			; Yup. Bomb immediately to prompt
				AND 	#$DF				; Convert to uppercase.
				CMP 	#"A"				; "A" Pressed.
				BEQ 	ChanA
				CMP 	#"B"				; "B" Pressed.
				BEQ 	ChanB
				CMP 	#"C"				; "C" Pressed.
				BEQ 	ChanC
				BRA 	GetInput			; Ignore all other input.
	
GoPrompt
				JSR 	Sound_ClearRegs		; Clear the sound chips
				JMP 	M_CmdPrompt			; Return to monitor

Toggle	; Toggle stereo value between left and right.
				LDA 	PotValue			; What's the current value?
				BEQ 	Right				; Zero? Then go flip it back to Right channel.
				STZ 	PotValue			; Must be $FF. Store Zero.
				BRA 	ToggleEnd
				
		Right	LDA 	#$FF				; Set it to $FF
				STA 	PotValue			; Store it.
				
	ToggleEnd	JSR 	SetPot				; Set the pot value.
				JMP 	GetInput			; Back to input.

CZero
				STZ 	Sound_ChipSelect	; Pick Chip Zero
				BRA 	ChanA1
	
COne
				STA 	Sound_ChipSelect	; Pick Chip One (anything not $0 means chip one. This is "A", so woot!)
				; Fallthrough!

ChanA1 			LDA 	#"A"				; Allows fallthrough from Chip Selects. Saves bytes.
	
ChanA	;Set Channel A to send out a tone
				JSR 	ChanGo				; Basic Channel Setup
				LDA 	#$F					; Amplitude level (100%)
				STA		Sound_Value
				LDA 	#8					; Amplitude Control Chan A	(Register 8)
				STA		Sound_Register		;
				JSR 	Sound_SendCmd

				;F#3
				LDA 	#$52				; Tone Freq A, fine tune
				STA		Sound_Value
				STZ 	Sound_Register		; Tone Generator Control (Register 0)
				JSR 	Sound_SendCmd
				LDA 	#1					; Tone Freq A, Coarse tune
				STA		Sound_Value
				INC 	Sound_Register		; Tone Generator Control (Register 1)
				JSR 	Sound_SendCmd
				JSR 	SetPot				; Set the potentiometer
				JMP 	GetInput			; We're done.

ChanB ;Set Channel B to send out a tone
				JSR 	ChanGo				; Basic Channel Setup
				LDA 	#$F					; Amplitude level (100%)
				STA		Sound_Value
				LDA 	#9					; Amplitude Control Chan B	(Register 9)
				STA		Sound_Register		;
				JSR 	Sound_SendCmd

				;C#4
				LDA 	#$E1				; Tone Freq B, fine tune
				STA		Sound_Value
				LDA		#2
				STA 	Sound_Register		; Tone Generator Control (Register 2)
				JSR 	Sound_SendCmd
				STZ		Sound_Value			; Tone Freq B, Coarse tune
				INC 	Sound_Register		; Tone Generator Control (Register 3)
				JSR 	Sound_SendCmd
				JSR 	SetPot				; Set the potentiometer
				JMP 	GetInput			; We're done.
				
				
				

ChanC ;Set Channel C to send out a tone
				JSR 	ChanGo				; Basic Channel Setup
				LDA 	#$F					; Amplitude level (100%)
				STA		Sound_Value
				LDA 	#10					; Amplitude Control Chan C	(Register 10)
				STA		Sound_Register		;
				JSR 	Sound_SendCmd

				;F#4
				LDA 	#$A9				; Tone Freq C, fine tune
				STA		Sound_Value
				LDA		#4
				STA 	Sound_Register		; Tone Generator Control (Register 4)
				JSR 	Sound_SendCmd
				STZ		Sound_Value			; Tone Freq C, Coarse tune
				INC 	Sound_Register		; Tone Generator Control (Register 5)
				JSR 	Sound_SendCmd
				JSR 	SetPot				; Set the potentiometer
				JMP 	GetInput			; We're done.

ChanGo
				STA 	CurrentChannel		; Why does this work? Because we always arrive here with the right value in Accumulator!
				JSR 	Sound_ClearRegs		; Clear the sound chips
				LDA 	#$38				; 00111000	 Disable noise and enable tone on all channels
				STA		Sound_Value
				LDA 	#7					; Mixer Control & I/O Enabler	(Register 7)
				STA 	Sound_Register
				JSR 	Sound_SendCmd
				RTS

SetPot
	LDA Sound_ChipSelect	; What Sound Chip are we on?
	BEQ ChipIsZero			; Zero, apparently. Go do that.
	ChipIsOne				; If chip one, do this.
 		LDA CurrentChannel	; Which Channel are we on?
		CMP #"A"			; A?
		BEQ OneA			; Go do A.
		CMP #"B"			; B?
		BEQ OneB			; Go do B.
	OneC					; Must be C.
		LDA #64				; PB6 / Sending #64
		STA PotChip			; ... to Sound_OutRegB means select Pot Chip (P2) for Both Chips Channel C
		LDA #%00010011 		; Select Pot 1
		BRA EndSetPot		; Done!
	OneA
		LDA #32				; PB5 / Sending #16 
		STA PotChip			; ... to Sound_OutRegB means select Pot Chip (P0) for Sound Chip 0, Ch. A or B
		LDA #%00000011 		; Select Pot 0
		BRA EndSetPot		; Done!
	OneB
		LDA #32				; PB5 / Sending #16 
		STA PotChip			; ... to Sound_OutRegB means select Pot Chip (P0) for Sound Chip 0, Ch. A or B
		LDA #%00010011 		; Select Pot 1
		BRA EndSetPot		; Done!
		
	ChipIsZero
		LDA CurrentChannel	; Which Channel are we on?
		CMP #"A"			; A?
		BEQ ZeroA			; Go do A.
		CMP #"B"			; B?
		BEQ ZeroB			; Go Do B.
	ZeroC					; Must be C.
		LDA #64				; PB6 / Sending #64 
		STA PotChip			; ... to Sound_OutRegB means select Pot Chip (P2) for Both Chips Channel C
		LDA #%00000011 		; Select Pot 0
		BRA EndSetPot		; Done!
	ZeroA
		LDA #16				; PB4 / Sending #16 
		STA PotChip			; ... to Sound_OutRegB means select Pot Chip (P0) for Sound Chip 0, Ch. A or B
		LDA #%00000011 		; Select Pot 0
		BRA EndSetPot		; Done!
	ZeroB
		LDA #16				; PB4 / Sending #16 
		STA PotChip			; .... to Sound_OutRegB means select Pot Chip (P0) for Sound Chip 0, Ch. A or B
		LDA #%00010011 		; Select Pot 1

EndSetPot
		STA PotSelect		; Done!
	
		LDA PotChip
		STA Sound_OutRegB	; Pull /CS/ low

		LDA PotSelect
		STA Sound_SR		; Send Command Byte
		JSR Sound_IRQLoop	; Wait for shift to finish

		LDA PotValue
		STA Sound_SR		; Send Panning Value
		JSR Sound_IRQLoop	; Wait for shift to finish.

		STZ Sound_OutRegB	; Pull /CS/ high
RTS

Sound_IRQLoop				; Checks if Sound_VIA done shifting bits.
	LDA Sound_IFR		
	BIT #%00000100			; Is the Shift Register Interupt Flag Set?
	BEQ Sound_IRQLoop		; Nope! Check again.
	RTS						; Yep! Shift done.